@@ -22,22 +22,23 @@
 
 #include "audacious-dumb.h"
 
-#include <audacious/output.h>
-
 #include <gtk/gtk.h>
 #include "interface.h"
 #include "support.h"
 
 DuhFile *duh_file = NULL;
-static GThread *decode_thread;
 static gboolean disable_amiga_mods = FALSE;
 static const gchar duh_custom_title_format[] = "${file-name}${?title: - ${title}}";
 static gint output_frequency;
-static const gint output_frequencies[]={4000, 8000, 11025, 22050, 44100, 48000, 64000, 96000};
+static const gint output_frequencies[]={22050, 44100, 48000, 64000, 96000, 192000};
 static GtkWidget *prefs;
 static GMutex *dumblock;
 
-gchar *fmts[] = { "mod", "s3m", "it", "xm", "duh", NULL };
+static GMutex *control_mutex;
+static GCond *control_cond;
+static gboolean stop_flag = FALSE;
+
+static const gchar *fmts[] = { "mod", "s3m", "it", "xm", "duh", NULL };
 
 InputPlugin duh_ip = {
     .description = "DUMB Plugin " VERSION,
@@ -45,26 +46,24 @@ InputPlugin duh_ip = {
     .cleanup = duh_cleanup,
     .about = duh_about,
     .configure = duh_configure,
-    .play_file = play_file,
+    .play = play_start,
     .stop = stop,
     .pause = duh_pause,
-    .seek = seek,
-    .get_song_info = get_song_info,
     .file_info_box = file_info_box,
-    .get_song_tuple = get_tuple_info,
     .is_our_file_from_vfs = is_our_file_from_vfs,
     .vfs_extensions = fmts,
     .mseek = mseek,
-    .probe_for_tuple = duh_probe_for_tuple
+    .probe_for_tuple = duh_probe_for_tuple,
+    .priority = 0 /* Would be nice to be above Modplug but minus values don't work */ 
 };
 
 InputPlugin *duh_iplist[] = { &duh_ip, NULL };
 
 SIMPLE_INPUT_PLUGIN(duh_ip, duh_iplist)
 
-static void duh_init(void)
+static gboolean duh_init(void)
 {
-	ConfigDb *db;
+	mcs_handle_t *db;
 	db = aud_cfg_db_open();
 
 	aud_cfg_db_get_bool(db, "DUMB", "disable_amiga_mods", &disable_amiga_mods);
@@ -72,7 +71,7 @@ static void duh_init(void)
 		aud_cfg_db_set_bool(db, "DUMB", "disable_amiga_mods", FALSE);
 
 	aud_cfg_db_get_int(db, "DUMB", "output_frequency", &output_frequency);	
-	if (output_frequency < 4000 || output_frequency > 96000) {
+	if (output_frequency < 22050 || output_frequency > 192000) {
 		output_frequency = 44100;
 		aud_cfg_db_set_int(db, "DUMB", "output_frequency", output_frequency);
 	}
@@ -86,36 +85,52 @@ static void duh_init(void)
 	dumb_it_max_to_mix = 256;
 	
 	dumblock = g_mutex_new();
+	control_mutex = g_mutex_new();
+	control_cond = g_cond_new();
+	
+	return TRUE;
 }
 
 static void duh_cleanup(void)
 {
 	g_mutex_free(dumblock);
+	g_mutex_free(control_mutex);
+	g_cond_free(control_cond);  
 }	
 
-static int is_our_file_from_vfs(gchar *filename, VFSFile *file)
+static int is_our_file_from_vfs(const gchar *filename, VFSFile *file)
 {
 	gchar magic[4];
 	const gchar *ext;
 
-	aud_vfs_fread(magic, 1, 4, file);
+	g_return_val_if_fail(filename != NULL, FALSE);
+	g_return_val_if_fail(file != NULL, FALSE);
+
+	vfs_fread(magic, 1, 4, file);
 	if (!memcmp(magic, XM_MAGIC, 4))
                 return TRUE;
 	if (!memcmp(magic, IT_MAGIC, 4))
 		return TRUE;
 
-	aud_vfs_fseek(file, 44, SEEK_SET);
-	aud_vfs_fread(magic, 1, 4, file);
+	vfs_fseek(file, 44, SEEK_SET);
+	vfs_fread(magic, 1, 4, file);
 	if (!memcmp(magic, S3M_MAGIC, 4))
 		return TRUE;
 
-	aud_vfs_fseek(file, 1080, SEEK_SET);
-	aud_vfs_fread(magic, 1, 4, file);   
+	vfs_fseek(file, 1080, SEEK_SET);
+	vfs_fread(magic, 1, 4, file);   
 	if (!memcmp(magic, MOD_MAGIC_FASTTRACKER6, 4))
 		return TRUE;
 	if (!memcmp(magic, MOD_MAGIC_FASTTRACKER8, 4))
 		return TRUE;
 
+/*
+	printf("Is %s our file?\n", filename);
+
+	if (disable_amiga_mods) {
+		printf("Amiga support is disabled\n");
+	}
+*/
 	if (!disable_amiga_mods) {
 		if (!memcmp(magic, MOD_MAGIC_PROTRACKER4, 4))
 			return TRUE;
@@ -163,20 +178,20 @@ static gint duh_universal_load_vfs(DUH **duh, const char *filename, VFSFile *ext
 	if (extfd)
 		fd = extfd;
 	else {
-		fd = aud_vfs_fopen(filename, "rb");
+		fd = vfs_fopen(filename, "rb");
 		g_return_val_if_fail(fd != NULL, -1);
 	}
 
-	filesize = aud_vfs_fsize(fd);
+	filesize = vfs_fsize(fd);
 	g_return_val_if_fail(filesize > 0, -1);
 	
 	filemap = malloc(filesize);
 	g_return_val_if_fail(filemap != NULL, -1);
 
-	readsize = aud_vfs_fread(filemap, 1, filesize, fd);
+	readsize = vfs_fread(filemap, 1, filesize, fd);
 
 	if (!extfd)
-		aud_vfs_fclose(fd);
+		vfs_fclose(fd);
 
 	if (readsize == 0) {
 		g_warning("audacious-dumb: Couldn't read from %s", filename);
@@ -215,7 +230,7 @@ static gint duh_universal_load_vfs(DUH **duh, const char *filename, VFSFile *ext
 	return ext_index;
 }
 
-static Tuple *get_tuple_info_from_duh(DUH *duh, char *filename)
+static Tuple *get_tuple_info_from_duh(DUH *duh, const gchar *filename)
 {
 	const gchar *mod_title_raw;
 	gchar *mod_title;
@@ -223,11 +238,11 @@ static Tuple *get_tuple_info_from_duh(DUH *duh, char *filename)
 	g_return_val_if_fail(filename != NULL, NULL);
 	g_return_val_if_fail(duh != NULL, NULL);
 
-	Tuple *tuple = aud_tuple_new_from_filename(filename);
+	Tuple *tuple = tuple_new_from_filename(filename);
 
-	aud_tuple_associate_string(tuple, FIELD_FORMATTER, NULL, duh_custom_title_format);
-	aud_tuple_associate_string(tuple, FIELD_QUALITY, NULL, "sequenced");
-	aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, (int)(duh_get_length(duh) * 1000LL >> 16) );
+	tuple_associate_string(tuple, FIELD_FORMATTER, NULL, duh_custom_title_format);
+	tuple_associate_string(tuple, FIELD_QUALITY, NULL, "sequenced");
+	tuple_associate_int(tuple, FIELD_LENGTH, NULL, (int)(duh_get_length(duh) * 1000LL >> 16) );
 	
 	mod_title_raw = duh_get_tag(duh, "TITLE");
 	if (mod_title_raw) {
@@ -239,7 +254,7 @@ static Tuple *get_tuple_info_from_duh(DUH *duh, char *filename)
 		g_strstrip(mod_title);
 
 		if (mod_title[0])
-			aud_tuple_associate_string(tuple, FIELD_TITLE, NULL, mod_title);
+			tuple_associate_string(tuple, FIELD_TITLE, NULL, mod_title);
 
 		g_free(mod_title); mod_title = NULL;
 	}
@@ -248,65 +263,30 @@ static Tuple *get_tuple_info_from_duh(DUH *duh, char *filename)
 	return tuple;
 }
 
-static Tuple *__get_tuple_info(char *filename, VFSFile *fd)
+static Tuple *duh_probe_for_tuple(const gchar *filename, VFSFile *fd)
 {
 	Tuple *input = NULL;
 	DUH *duh = NULL;
 	int ext_index;
 
 	g_return_val_if_fail(filename != NULL, NULL);
+	g_return_val_if_fail(fd != NULL, NULL);
+
+	if (!is_our_file_from_vfs(filename, fd))
+		return NULL;
+		
+	vfs_rewind(fd);
 
 	ext_index = duh_universal_load_vfs(&duh, filename, fd, UNIREAD_ALL);
 	if (ext_index == -1)
 		return NULL; /* Return empty Tuple if module was not loadable */
 	
 	input = get_tuple_info_from_duh(duh, filename);
-	aud_tuple_associate_string(input, FIELD_CODEC, NULL, uniread[ext_index].description);	
-
+	tuple_associate_string(input, FIELD_CODEC, NULL, uniread[ext_index].description);	
+		
 	return input;
 }
 
-static Tuple *get_tuple_info(char *filename)
-{
-	return __get_tuple_info(filename, NULL);
-}
-
-static Tuple *duh_probe_for_tuple(char *filename, VFSFile *fd)
-{
-	if (!is_our_file_from_vfs(filename, fd))
-		return NULL;
-		
-	aud_vfs_rewind(fd);
-		
-	return __get_tuple_info(filename, fd);
-}
-
-static void get_song_info(char *filename, char **title, int *length)
-{
-	Tuple *tuple = get_tuple_info(filename);
-	
-        (*length) = -1;
-        (*title) = NULL;
-	
-	*length = aud_tuple_get_int(tuple, FIELD_LENGTH, NULL);
-	*title = aud_tuple_formatter_process_string(tuple, duh_custom_title_format);
-
-	tuple_free(tuple);
-}
-
-static gchar *get_title(DUH *duh, char *filename)
-{
-        Tuple *tuple = NULL;
-	gchar *title;
-	
-	tuple = get_tuple_info_from_duh(duh, filename);
-	title = aud_tuple_formatter_process_string(tuple, duh_custom_title_format);
-
-	aud_tuple_free(tuple);
-
-	return title;
-}
-
 static void install_callbacks(DUH_SIGRENDERER *sr)
 {
 	DUMB_IT_SIGRENDERER *itsr = duh_get_it_sigrenderer(sr);
@@ -314,85 +294,29 @@ static void install_callbacks(DUH_SIGRENDERER *sr)
 	dumb_it_set_xm_speed_zero_callback(itsr, &dumb_it_callback_terminate, NULL);
 }
 
-static void *play_loop(void *arg)
+static gboolean play_start(InputPlayback *playback, const gchar *filename,
+ VFSFile *file, gint start_time, gint stop_time, gboolean pause)
 {
-	char data[2048 * 2];
-	int blk_size, rate;
-	unsigned int bytes;
-	int actual_read;
-	
-	InputPlayback *playback = arg;
-
-	blk_size = 512 * (duh_file->bits_per_sample / 8) * duh_file->channels;
-	rate = duh_file->samples_per_sec * duh_file->channels * (duh_file->bits_per_sample / 8);
-	while (playback->playing)
-	{
-		if (duh_file->seek_to != -1)
-		{
-			duh_file->position = (unsigned long)((gint64)duh_file->seek_to * (gint64)rate / 1000L);
-			duh_end_sigrenderer(duh_file->file);
-			duh_file->file = duh_start_sigrenderer(duh_file->duh, 0, duh_file->channels, (unsigned long)((gint64)duh_file->seek_to << 16) / 1000L);
-			install_callbacks(duh_file->file);
-			playback->output->flush(duh_file->seek_to);
-			duh_file->seek_to = -1;
-		}
-
-		if (!playback->eof)
-		{
-			bytes = blk_size;
-			if (duh_file->length - duh_file->position < bytes)
-				bytes = duh_file->length - duh_file->position;
-			if (bytes > 0)
-			{
-				actual_read = duh_render(duh_file->file, 16, 0, 1.0f, 65536.0f / duh_file->samples_per_sec, bytes / ((duh_file->bits_per_sample / 8) * duh_file->channels), data);
-				actual_read *= (duh_file->bits_per_sample / 8) * duh_file->channels;
-
-				if (actual_read == 0)
-					playback->eof = TRUE;
-				else
-				{
-					if (playback->playing && duh_file->seek_to == -1)
-						playback->pass_audio(playback, (duh_file->bits_per_sample == 16) ? FMT_S16_LE : FMT_U8,
-						duh_file->channels, bytes, data, &playback->playing);
-
-					duh_file->position += actual_read;
-				}
-			}
-			else
-				playback->eof = TRUE;
-		}
-		else
-		{
-			playback->output->buffer_free ();
-			playback->output->buffer_free ();
-			while (playback->output->buffer_playing())
-				g_usleep(10000);
-			playback->playing = 0;
-		} 
-	}
-
-	duh_end_sigrenderer(duh_file->file);
-	unload_duh(duh_file->duh);
-	return NULL;
-}
-
-static void play_file(InputPlayback *playback)
-{
-	char *name;
+	Tuple *tuple;
 	int rate;
-	char *filename = playback->filename;
 
-	playback->error = FALSE;
+	const int buffer_size = 16 * 1024;
+	char *buffer;
+	int actual_read, render_size;
+	float delta, volume;
+
+	if (file == NULL)
+		return FALSE;
 
 	duh_file = g_malloc(sizeof (DuhFile));
 	memset(duh_file, 0, sizeof (DuhFile));
 
-	duh_universal_load_vfs(&duh_file->duh, filename, NULL, UNIREAD_ALL);
+	duh_universal_load_vfs(&duh_file->duh, filename, file, UNIREAD_ALL);
 	if (!duh_file->duh) {
 		g_warning("audacious-dumb: Unable to play %s", filename);
 		g_free(duh_file);
 		duh_file = NULL;
-		return;
+		return FALSE;
 	}
 
 	if ((duh_file->file = duh_start_sigrenderer(duh_file->duh, 0, 2, 0)))
@@ -401,64 +325,135 @@ static void play_file(InputPlayback *playback)
 		duh_file->channels = 2;
 		duh_file->samples_per_sec = output_frequency;
 		duh_file->bits_per_sample = 16;
-		duh_file->length = (long long)duh_get_length(duh_file->duh) * (output_frequency * 2 * 2) >> 16;
-		duh_file->position = 0;
-		playback->playing = 1;
 
 		if (playback->output->open_audio((duh_file->bits_per_sample == 16) ? FMT_S16_LE : FMT_U8, duh_file->samples_per_sec, duh_file->channels) == 0)
 		{
-			playback->error = TRUE;
 			duh_end_sigrenderer(duh_file->file);
 			unload_duh(duh_file->duh);
 			g_free(duh_file);
 			duh_file = NULL;
-			return;
+			return FALSE;
 		}
-		name = get_title(duh_file->duh, filename);
+
+		if (pause)
+			playback->output->pause(TRUE);
+		
 		rate = duh_file->samples_per_sec * duh_file->channels * (duh_file->bits_per_sample / 8);
-		playback->set_params(playback, name, 1000LL * duh_file->length / rate, 8 * rate, duh_file->samples_per_sec, duh_file->channels);
-		g_free(name);
-		duh_file->seek_to = -1;
-		decode_thread = g_thread_self();
+		tuple = get_tuple_info_from_duh(duh_file->duh, filename);
+		playback->set_tuple(playback, tuple);
+		playback->set_params(playback, 8 * rate, duh_file->samples_per_sec, duh_file->channels);
+
+		g_mutex_lock(control_mutex);
+		duh_file->seek_to = (start_time > 0) ? start_time : -1;
+		stop_flag = FALSE;
 		playback->set_pb_ready(playback);
-		play_loop(playback);
+		g_mutex_unlock(control_mutex);
+
+		/* Values taken from the dumbout.c example */
+		delta = 65536.0f / duh_file->samples_per_sec;
+		volume = 1.0f;
+
+		buffer = g_malloc(buffer_size);
+
+		/* The docs say output buffer equals render_size * channels (2)
+		 * but it seems not enough so we're keeping the old calculated value (4)
+		 */
+		render_size = buffer_size / ((duh_file->bits_per_sample / 8) * duh_file->channels);
+
+		while (!stop_flag)
+		{
+			g_mutex_lock(control_mutex);
+			if (duh_file->seek_to != -1)
+			{
+				duh_end_sigrenderer(duh_file->file);
+				duh_file->file = duh_start_sigrenderer(duh_file->duh, 0, duh_file->channels, (unsigned long)((gint64)duh_file->seek_to << 16) / 1000L);
+				install_callbacks(duh_file->file);
+				playback->output->flush(duh_file->seek_to); 
+				duh_file->seek_to = -1;
+				g_cond_signal(control_cond);
+			}
+
+			g_mutex_unlock(control_mutex);
+
+			if (!duh_file->eof)
+			{
+				actual_read = duh_render(duh_file->file, duh_file->bits_per_sample, 0, volume, delta, render_size, buffer);
+				actual_read *= (duh_file->bits_per_sample / 8) * duh_file->channels;
+	
+				if (actual_read == 0)
+					duh_file->eof = TRUE;
+				else
+				{
+					if (!stop_flag && duh_file->seek_to == -1)
+						playback->output->write_audio(buffer, actual_read);
+				}
+			}
+			else
+			{
+				while (!stop_flag && playback->output->buffer_playing())
+					g_usleep(20000);
+			
+				break;
+			} 
+		}
+
+		playback->output->close_audio();
+
+		g_mutex_lock(control_mutex);
+		stop_flag = TRUE;
+		duh_file->eof = TRUE;
+		g_mutex_unlock(control_mutex);
+
+		duh_end_sigrenderer(duh_file->file);
+		unload_duh(duh_file->duh);
+		g_free(buffer);
+		g_free(duh_file);
+		duh_file = NULL;
+
+		return TRUE;
 	}
+	return FALSE;
 }
 
 static void stop(InputPlayback *playback)
 {
-	if (duh_file && playback->playing)
+	g_mutex_lock(control_mutex);
+
+	if (!stop_flag)
 	{
-		playback->playing = 0;
-		g_thread_join(decode_thread);
-		playback->output->close_audio();
-		g_free(duh_file);
-		duh_file = NULL;
+		stop_flag = TRUE;
+		playback->output->abort_write();
+		g_cond_signal(control_cond);
 	}
+
+	g_mutex_unlock (control_mutex);
 }
 
-static void duh_pause(InputPlayback *playback, short p)
+static void duh_pause(InputPlayback *playback, gboolean pause)
 {
-	playback->output->pause(p);
+	g_mutex_lock(control_mutex);
+
+	if (!stop_flag)
+		playback->output->pause(pause);
+            
+	g_mutex_unlock(control_mutex);
 }
 
-static void mseek(InputPlayback *playback, gulong millisecond)
+static void mseek(InputPlayback *playback, gint time)
 {
-	duh_file->seek_to = millisecond;
+	g_mutex_lock(control_mutex);
 
-	playback->eof = FALSE;
-
-	while (duh_file->seek_to != -1)
-		g_usleep(10000);
+	if (!stop_flag) {
+		duh_file->seek_to = time;
+		duh_file->eof = FALSE;
+	        g_cond_signal(control_cond);
+                g_cond_wait(control_cond, control_mutex);
+	}
+	
+	g_mutex_unlock(control_mutex);
 }
 
-static void seek(InputPlayback *playback, int time)
-{
-	gulong millisecond = time * 1000;
-	mseek(playback, millisecond);
-}
-
-static void file_info_box(char *filename)
+static void file_info_box(const gchar *filename)
 {
 	GtkWidget *songinfo;
 	GtkStyle *style = NULL;
@@ -583,7 +578,7 @@ static void duh_configure(void) {
 }
 
 static void duh_configure_ok_cb(GtkButton *w, gpointer data) {
-	ConfigDb *db;
+	mcs_handle_t *db;
 	gint output_frequency_idx;
 
 	db = aud_cfg_db_open();
@@ -601,19 +596,21 @@ static void duh_configure_ok_cb(GtkButton *w, gpointer data) {
 
 static void duh_about(void) {
 	static GtkWidget *about;
+	gchar *about_text;
+	
+	if (about)
+		return;
 
-	if (about) return;
+        about_text = g_strjoin("", _("DUMB Input Plugin "), VERSION,
+				   _("\n"
+				     "by Christian Birchinger <joker@netswarm.net>\n"
+				     "\n"
+				     "Based on the original XMMS plugin by Ben Davis\n"
+				     "and the BMP port by Michael Doering\n"
+				     "\n"
+				     "Built with DUMB version "), DUMB_VERSION_STR, NULL);
+                                                                                
+	audgui_simple_message (& about, GTK_MESSAGE_INFO, "About DUMB Plugin", about_text);
 
-	about = audacious_info_dialog("About DUMB Plugin",
-				  "DUMB Input Plugin " VERSION "\n"
-				  "by Christian Birchinger <joker@netswarm.net>\n"
-				  "\n"
-				  "Based on the original XMMS plugin by Ben Davis\n"
-				  "and the BMP port by Michael Doering\n"
-				  "\n"
-				  "Built with DUMB version " DUMB_VERSION_STR,
-				  "Ok", FALSE, NULL, NULL);
-
-	g_signal_connect(G_OBJECT(about), "destroy",
-			 G_CALLBACK(gtk_widget_destroyed), &about);
+	g_free(about_text);
 }
