@@ -29,7 +29,7 @@
 #error not have the thread safety required by this plug-in.
 #endif
 
-#define VERSION "0.57"
+#define VERSION "0.65"
 #define VERSIONSTRING "DUMB Audio Plugin " VERSION
 
 #include <pthread.h>
@@ -38,9 +38,10 @@
 #include <string.h>
 #include <stdio.h>
 
+#include <audacious/configdb.h>
 #include <audacious/plugin.h>
-#include <audacious/util.h>
-#include <audacious/main.h>
+#include <libaudgui/libaudgui.h>
+#include <libaudgui/libaudgui-gtk.h>
 
 extern InputPlugin duh_ip;
 
@@ -50,26 +51,22 @@ typedef struct
 	DUH_SIGRENDERER *file;
 	short channels, bits_per_sample;
 	long samples_per_sec;
-	unsigned long position, length;
 	int seek_to;
+	int eof;
 	pid_t pid;
 }
 DuhFile;
 
-static void duh_init(void);
+static gboolean duh_init(void);
 static void duh_cleanup(void);
-static int is_our_file_from_vfs(gchar *filename, VFSFile *file);
-
-static void play_file(InputPlayback *data);
+static int is_our_file_from_vfs(const gchar *filename, VFSFile *file);
+static gboolean play_start(InputPlayback *playback, const gchar *filename, VFSFile *file, gint start_time, gint stop_time, gboolean pause);
 static void stop(InputPlayback *data);
-static void seek(InputPlayback *data, int time);
-static void mseek(InputPlayback *data, gulong millisecond);
-static void duh_pause(InputPlayback *data, short p);
+static void mseek(InputPlayback *data, gint time);
+static void duh_pause(InputPlayback *data, gboolean p);
 
-static Tuple *get_tuple_info(char *filename);
-static Tuple *duh_probe_for_tuple(char *filename, VFSFile *fd);
-static void get_song_info(char *filename, char **title, int *length);
-static void file_info_box(char *filename);
+static Tuple *duh_probe_for_tuple(const gchar *filename, VFSFile *fd);
+static void file_info_box(const gchar *filename);
 static void duh_about(void);
 static void duh_configure(void);
 static void duh_configure_ok_cb(GtkButton *w, gpointer data);
